#ifdef SCCS
static char sccsid[]="@(#)button.c	1.2 Stellar 89/03/22";
#endif
/*
			Copyright 1989 by
			Stellar Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stellar Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
*/
/*
 * LUI Button routines
 *
 */
#include <stdio.h>

#define LUI_BUTTON_DECLARE
#include "lui.h"
#include "pixmaps.h"


/******************************************************************************/
 LUI_BUTTON_PAD *
LUI_ButtonPadOpenType(name, type, parent, x, y, border_size, background_color)
    char *name;
    Window parent;
    int type, x, y, border_size;
    unsigned long background_color;
{
    LUI_BUTTON_PAD *button_pad;
    int w, h;
    char *path, *LUI_GetPathName();

    button_pad = LUI_ButtonPadOpen(name, parent, x, y, border_size, background_color);
    button_pad->type = type;

    switch(type) {
      case LUI_BUTTON_RADIO:
/* Replaced:
	path = LUI_GetPathName(LUI_BUTTON_RADIO_PIXMAP_IN);
	button_pad->pixmap_in  = LUI_ReadPixmap(path, 0, 0, &w, &h);
	free(path);
	path = LUI_GetPathName(LUI_BUTTON_RADIO_PIXMAP_OUT);
	button_pad->pixmap_out = LUI_ReadPixmap(path, 0, 0, &w, &h);
	free(path);
   With:
*/
        button_pad->pixmap_in = LUI_PixmapRadioIn;
        button_pad->pixmap_out = LUI_PixmapRadioOut;
/* By:  Brian Paul  On:  3-15-91 */
	break;
      case LUI_BUTTON_OPTION:
/* Replaced:
	path = LUI_GetPathName(LUI_BUTTON_OPTION_PIXMAP_IN);
	button_pad->pixmap_in  = LUI_ReadPixmap(path,  0, 0, &w, &h);
	free(path);
	path = LUI_GetPathName(LUI_BUTTON_OPTION_PIXMAP_OUT);
	button_pad->pixmap_out = LUI_ReadPixmap(path, 0, 0, &w, &h);
	free(path);
   With:
*/
        button_pad->pixmap_in = LUI_PixmapOptionIn;
        button_pad->pixmap_out = LUI_PixmapOptionOut;
/* By:  Brian Paul  On:  3-15-91 */
	break;
    }

    return(button_pad);
}

/******************************************************************************/
 LUI_BUTTON_PAD *
LUI_ButtonPadOpen(name, parent, x, y, border_size, background_color)
    char *name;
    Window parent;
    int x, y, border_size;
    unsigned long background_color;
{
    LUI_BUTTON_PAD *button_pad;
    int LUI_ButtonPadDraw();

    /* Initialize placement variables */
    LUI_ButtonPadWidth  = 0;
    LUI_ButtonPadHeight = 0;

    /* Create the button pad window */
    LUI_ButtonWindow = LUI_CreateWindow(parent, 0, 0);

    XSetWindowBackground(LUI_Display, LUI_ButtonWindow, background_color);

    /* Set the cursor */
    LUI_Cursor(LUI_ButtonWindow, LUI_MAIN_CURSOR);

    /* Add Expose event */
    LUI_EventAdd(LUI_ButtonWindow, ExposureMask, LUI_ButtonPadDraw);

    /* Allocate the button pad */
    if (!(button_pad = (LUI_BUTTON_PAD *)malloc(sizeof(LUI_BUTTON_PAD)))) {
	printf("LUI_ButtonPadOpen(): can't allocate button pad %s\n", name);
	return(NULL);
    }

    /* Set Current button pad */
    LUI_ButtonPad = button_pad;

    /* Setup button pad variables */
    strcpy(button_pad->name, name);
    button_pad->type        = 0;
    button_pad->x           = x;
    button_pad->y           = y;
    button_pad->window      = LUI_ButtonWindow;
    button_pad->border_size = border_size; 
    button_pad->hue         = LUI_DefaultHue;
    button_pad->background_color = background_color;

    /* Add button pad to head of list */
    button_pad->np = LUI_ButtonPadHead;
    LUI_ButtonPadHead  = button_pad;

    return(button_pad);
}


/******************************************************************************/
/* Added on 5-15-92 by BEP, allows buttons to be added to existing pad. */
LUI_BUTTON_PAD *LUI_ButtonPadReopen(name)
char *name;
{
   LUI_BUTTON_PAD *button_pad;
   int LUI_ButtonPadDraw();
   register LUI_BUTTON_PAD *bp;

   /* find the button pad data struct */
   for (bp = LUI_ButtonPadHead; bp; bp = bp->np) {
      if (strcmp(bp->name, name)==0) {
	 LUI_ButtonPad = bp;
	 break;
      }
   }
   if (!LUI_ButtonPad) {
      return NULL;
   }

   LUI_ButtonWindow = bp->window;
   LUI_ButtonPadWidth = bp->width - bp->border_size;
   LUI_ButtonPadHeight = bp->height - bp->border_size;;

   return bp;
}



/******************************************************************************/
LUI_ButtonPadClose(name)
    char *name;
{
    int x, y, width, height;

    /* Check for errors */
    if (!LUI_ButtonPad) {
	fprintf(stderr, "LUI_ButtonPadClose: Can't find open ButtonPad (%s)!\n", name);
	exit(-1);
    }

    /* Set width and height */
    LUI_ButtonPad->width  = LUI_ButtonPadWidth  + LUI_ButtonPad->border_size;
    LUI_ButtonPad->height = LUI_ButtonPadHeight + LUI_ButtonPad->border_size; /* + 1; */

    /* Reset size of window and reposition it */
    x      = LUI_ButtonPad->x;
    y      = LUI_ButtonPad->y;
    width  = LUI_ButtonPad->width;
    height = LUI_ButtonPad->height;
    XMoveResizeWindow(LUI_Display, LUI_ButtonWindow, x, y, width, height);

    /* Reset Button Pad Globals */
    LUI_ButtonPad = NULL;
}

/******************************************************************************/
LUI_ButtonPadColor(name, hue)
    char *name;
    float hue;
{
    register LUI_BUTTON_PAD *bp;

    for (bp = LUI_ButtonPadHead; bp; bp = bp->np) 
      if (!strcmp(bp->name, name)) break;

    if (bp) 
      bp->hue = hue;
}

/******************************************************************************/
LUI_ButtonPadMove(name, x, y)
    char *name;
    int x, y;
{
    register LUI_BUTTON_PAD *bp;

    for (bp = LUI_ButtonPadHead; bp; bp = bp->np) 
      if (!strcmp(bp->name, name)) {
	  XMoveWindow(LUI_Display, bp->window, x, y);
	  return(1);
      }

    return(0);
}

/******************************************************************************/
LUI_ButtonPadVisible(name, on_off)
    char *name;
    int on_off;
{
    register LUI_BUTTON_PAD *bp;

    for (bp = LUI_ButtonPadHead; bp; bp = bp->np) 
      if (!strcmp(bp->name, name)) break;

    if (bp) {
	if (on_off) {
	  XMapWindow  (LUI_Display, bp->window);
	  bp->visible = 1;
        }
	else {
	  XUnmapWindow(LUI_Display, bp->window);
	  bp->visible = 0;
        }
    }
    else 
	printf("LUI_ButtonPadVisible: can't find ButtonPad %s\n", name);
}

/******************************************************************************/
 LUI_BUTTON_PAD *
LUI_ButtonPadLookup(window)
    Window window;
{
    register LUI_BUTTON_PAD *bp;

    for (bp = LUI_ButtonPadHead; bp; bp = bp->np) 
	if (bp->window == window) return(bp);

    return(NULL);
}
   
/******************************************************************************/
LUI_ButtonPadQuery(name, x, y, width, height)
    char *name;
    int *x, *y, *width, *height;
{
    register LUI_BUTTON_PAD *bp;

    for (bp = LUI_ButtonPadHead; bp; bp = bp->np) 
	if (!strcmp(bp->name, name)) break;

    if (bp) {
	*x      = bp->x;
	*y      = bp->y;
	*width  = bp->width;
	*height = bp->height;
    }
    else 
      fprintf(stderr, "LUI_ButtonPadQuery: can't find ButtonPad %s\n", name);
}

/******************************************************************************/
LUI_ButtonPadDraw(window, event)
    Window window;
    XEvent *event;
{
    LUI_BUTTON_PAD *bp;

    /* Squelch multiple redraws */
    if (event->xexpose.count > 0)
      return;

    if (!(bp = LUI_ButtonPadLookup(window)))
      return;

    LUI_FrameWindow2(window, bp->width, bp->height, bp->border_size );
}

/******************************************************************************/
 LUI_BUTTON *
LUI_ButtonCreateType(name, x, y, size,index, func)
    char *name;
    int x, y, size;
/******* added by Marief, June 21st 1989 ********/
    int index;
    LUI_FNCP func;
{
    LUI_BUTTON *button;

    button = LUI_ButtonCreate(name, LUI_ButtonPad->type, x, y, size,index, func);

    return(button);
}

/******************************************************************************/
 LUI_BUTTON *
LUI_ButtonCreate(name, type, x, y,size,index,func)
    char *name;
    int type, x, y, size;
/******* added by Marief, June 21st 1989 ********/
    int index ;
    LUI_FNCP func;
{
    LUI_LABEL *label;
    register LUI_BUTTON *button;
    Window window;
    int width, height, LUI_ButtonProcess(), LUI_DrawRadioButton(), LUI_DrawOptionButton();
    char name_str[128];
    XColor xcol;

    /* Get the size of the window */
    height = LUI_BUTTON_YS - 10;
    width  = size;

/******* added by Marief, June 21st 1989 ********/
    if (index==NULL) index= 0;

    /* Adjust position based on Button size */
    x = x * width  + LUI_ButtonPad->border_size;
    y = y * height + LUI_ButtonPad->border_size; /* + 1; */

    /* Adjust max width and height of Button Pad window */
    if ((x + width ) > LUI_ButtonPadWidth ) LUI_ButtonPadWidth  = x + width;
    if ((y + height) > LUI_ButtonPadHeight) LUI_ButtonPadHeight = y + height;

    /* Make a label */
    if (LUI_ButtonPad->type == LUI_BUTTON_RADIO || 
	LUI_ButtonPad->type == LUI_BUTTON_OPTION) 
      sprintf(name_str, "  %s", name);
    else 
      sprintf(name_str, "%s", name);

    label = LUI_LabelOpen(name, 1, (char **)LUI_MakeLabel(name_str), 
			      LUI_ButtonWindow, 
			      x, y, width, height, 
			      10, LUI_ButtonProcess);
    LUI_LabelClose(name);
    XSetWindowBackground(LUI_Display, label->window, LUI_ButtonPad->background_color);

    LUI_LabelVisible(name, 1);
    LUI_LabelSetValue(0.75);
    LUI_LabelColor(name, LUI_DefaultHue2);
    label->color[LUI_LABEL_COL_FRONT] = LUI_Color_md_grey;
    LUI_LabelHighlight(name, 0);
    LUI_LabelDropShadow(name, 1);

    if (type == LUI_BUTTON_RADIO)
      LUI_LabelSetDrawFunc(name, LUI_DrawRadioButton);
    else if (type == LUI_BUTTON_OPTION)
      LUI_LabelSetDrawFunc(name, LUI_DrawOptionButton);

    /* Allocate a new button */
    if (!(button = (LUI_BUTTON *)malloc(sizeof(LUI_BUTTON)))) {
	fprintf(stderr, "LUI_ButtonCreate: can't allocate new button\n");
	return(NULL);
    }

    /* Initialize button */
    strcpy(button->label, name);
    button->state      = LUI_BUTTON_OUT;
    button->active     = 1;
    button->group      = type;
    button->x          = x;
    button->y          = y;
    button->width      = width;
    button->height     = height;
    /******* added by Marief, June 21st 1989 ********/
    button->index      = index;

    /* BEP 12-18-91: */
    if (LUI_ColorHilite) {
       /* non-default */
       if (LUI_Depth==24) {
          button->hicolor = LUI_ColorHilite;
       }
       else {
	  unsigned long int plane_masks[1];
	  if (XAllocColorCells( LUI_Display, LUI_Colormap, 0,
			       plane_masks, 0,
			       &button->hicolor, 1 )) {
	     xcol.red   = (LUI_ColorHilite>>8) & 0xff00;
	     xcol.green = (LUI_ColorHilite   ) & 0xff00;
	     xcol.blue  = (LUI_ColorHilite<<8) & 0xff00;
	     xcol.flags = DoRed | DoGreen | DoBlue;
	     xcol.pixel = button->hicolor;
	     XStoreColors( LUI_Display, LUI_Colormap, &xcol, 1 );
	  }
	  else {
	     printf("Xalloc failed\n");
	     button->hicolor = 0;
	  }
       }
    }
    else {
       button->hicolor = 0;
    }
    LUI_ColorHilite = 0;

    button->func       = func;
    button->window     = label->window;
    button->button_pad = LUI_ButtonPad;
    button->pixmap_in  = LUI_ButtonPad->pixmap_in;
    button->pixmap_out = LUI_ButtonPad->pixmap_out;
    label->owner       = (char *)button;

    /* Add button to head of list */
    button->np = LUI_ButtonHead;
    LUI_ButtonHead = button;

    return(button);
}

/******************************************************************************/
LUI_ButtonChangeLabel(button, name)
    LUI_BUTTON *button;
    char *name;
{
    LUI_LABEL *label;

    label = LUI_LabelLookupWindow(button->window);
    
    LUI_LabelChangeText(label->name, 1, (char **)LUI_MakeLabel(name));
}

/******************************************************************************/
LUI_ButtonDraw(bp)
    LUI_BUTTON *bp;
{
    LUI_LABEL *label;
    XColor xcol;

    label = LUI_LabelLookupWindow(bp->window);

/**** Modified by Marief May 24 1989****/
    if (!(bp->state)) {
	LUI_LabelSetValue(1.0);
	LUI_LabelColor(label->name, LUI_DefaultHue);
	label->color[LUI_LABEL_COL_FRONT] = LUI_Color_md_grey;
	label->border_size = 5;
	label->text_drop_shadow = 0;
    }
    else {
	LUI_LabelSetValue(0.75);
	LUI_LabelColor(label->name, LUI_DefaultHue2);

        /* BEP 12-18-91: */
	if (LUI_Depth==24) {
	   if (LUI_ColorHilite) {
	      bp->hicolor = LUI_ColorHilite;
	      label->color[LUI_LABEL_COL_HILITE1] = bp->hicolor;
	   }
	   else {
	      /* use default, white */
	      label->color[LUI_LABEL_COL_HILITE1] = LUI_Color_white;
	   }	      
	}
	else {
	   if (bp->hicolor && LUI_ColorHilite) {
	      xcol.red   = (LUI_ColorHilite>>8) & 0xff00;
	      xcol.green = (LUI_ColorHilite   ) & 0xff00;
	      xcol.blue  = (LUI_ColorHilite<<8) & 0xff00;
	      xcol.flags = DoRed | DoGreen | DoBlue;
	      xcol.pixel = bp->hicolor;
	      XStoreColors( LUI_Display, LUI_Colormap, &xcol, 1 );
	      label->color[LUI_LABEL_COL_HILITE1] = bp->hicolor;
	   }
	   else {
	      /* use default, white */
	      label->color[LUI_LABEL_COL_HILITE1] = LUI_Color_white;
	   }
	}

/**** Modified by Marief May 24 1989****/
/**** Updates the hilite color****/
/*	label->color[LUI_LABEL_COL_HILITE1] = LUI_ColorHilite;*/
	label->border_size = 10;
	label->text_drop_shadow = 0;
    }
    LUI_ColorHilite = 0;
    
    LUI_LabelHighlight(label->name, bp->state);
}

/******************************************************************************/
 LUI_BUTTON *
LUI_ButtonLookupWindow(window)
    Window window;
{
    register LUI_BUTTON *bp;

    for (bp = LUI_ButtonHead; bp; bp = bp->np) 
	if (bp->window == window)  
	  return(bp);

    return(NULL);
}
   
/******************************************************************************/
 LUI_BUTTON *
LUI_ButtonLookupName(name)
    char *name;
{
    register LUI_BUTTON *bp;

    for (bp = LUI_ButtonHead; bp; bp = bp->np) 
	if (!(strcmp(bp->label, name))) 
	  return(bp);

    return(NULL);
}
   
/******************************************************************************/
 static
LUI_ButtonProcess(label)
    LUI_LABEL *label;
{
    XEvent *event;
    Window window;
    register LUI_BUTTON_PAD *button_pad;
    register LUI_BUTTON *button, *bp;

    for (button = LUI_ButtonHead; button; button = button->np) 
	if (button->window == label->window)
	  break;

    if (!button) return;
    if (!button->active) return;

    event = label->event;
    button->event = label->event;
    button_pad = button->button_pad;

    switch (event->type) {
      case Expose:
	LUI_ButtonDraw(button);
	break;

      case EnterNotify:
	if (button->state == LUI_BUTTON_OUT)
	  LUI_LabelHighlightText(label->name, 1);
	break;

      case LeaveNotify:
	if (button->state == LUI_BUTTON_OUT)
	  LUI_LabelHighlightText(label->name, 0);
	break;

      case ButtonPress:

	switch (event->xbutton.button) {
	  case Button1:
	  case Button2:
	  case Button3:

	    /* RADIO BUTTON : only current selection is on */
	    if (button->group == LUI_BUTTON_RADIO) {
		/* Will be on - turn off all other radio buttons */
		if (button->state == LUI_BUTTON_OUT) {
		    /* Find all buttons selected for this group */
		    for (bp = LUI_ButtonHead; bp; bp = bp->np) {
			if (bp->group == button->group && 
			    bp->state == LUI_BUTTON_IN &&
			    bp->button_pad == button_pad) {

				/* Automatically turn it off */
				bp->state = LUI_BUTTON_OUT;
				LUI_ButtonDraw(bp);
			    }
		    }
		    /* Turn this button on */
		    button->state = LUI_BUTTON_IN;

		    /* Draw this button */
		    LUI_ButtonDraw(button);

		    /* Dispatch this button's function */
		    if (button->func) 
		      (*(button->func))(button);
		}
		
		/* Will be off */
		else 
		  LUI_ButtonDraw(button);
	    }

	    /* Just like a toggle */
	    else if (button->group == LUI_BUTTON_OPTION) {
		if (button->state == LUI_BUTTON_OUT) button->state = LUI_BUTTON_IN;
		else button->state = LUI_BUTTON_OUT;
		
		LUI_ButtonDraw(button);

		/* Dispatch function */
		if (button->func) 
		  (*(button->func))(button);
	    }

    	    /* If oneshot, press in now, out after function */
	    else if (button->group == LUI_BUTTON_ONESHOT) {
		XSync(LUI_Display, 0);
		button->state = LUI_BUTTON_IN;
		LUI_ButtonDraw(button);

		XSync(LUI_Display, 0);
		if (button->func)
		  (*(button->func))(button);

		XSync(LUI_Display, 0);
		button->state = LUI_BUTTON_OUT;
		LUI_ButtonDraw(button);
	    }

	    /* If group is other than 0 */
	    else if (button->group == LUI_BUTTON_FUNCTION) {
		/* Will be on - turn off all other radio buttons */
		if (button->state == LUI_BUTTON_OUT) {
		    /* Find all buttons selected for this group */
		    for (bp = LUI_ButtonHead; bp; bp = bp->np) {
			if (bp->group == button->group && 
			    bp->state == LUI_BUTTON_IN &&
			    bp->button_pad == button_pad) {

				/* Automatically turn it off */
				bp->state = LUI_BUTTON_OUT;
				LUI_ButtonDraw(bp);
			    }
		    }
		    /* Turn this button on */
		    button->state = LUI_BUTTON_IN;

		    /* Draw this button */
		    LUI_ButtonDraw(button);

		    /* Dispatch this button's function */
		    if (button->func) 
		      (*(button->func))(button);
		}
		
		/* Will be off */
		else 
		  LUI_ButtonDraw(button);
	    }

	    else if (button->group == LUI_BUTTON_TOGGLE) {
		if (button->state == LUI_BUTTON_OUT) button->state = LUI_BUTTON_IN;
		else button->state = LUI_BUTTON_OUT;
		

/**** Modified by Marief May 24 1989****/
/**** switch of button->func and LUI_ButtonDraw to allow the application to ***/
/**** change the button draw -- color for instance***/
		/* Dispatch function */
		if (button->func) 
		  (*(button->func))(button);

		LUI_ButtonDraw(button);
	    }

	    else if (button->group == LUI_BUTTON_CONFIG) {
		if (button->state == LUI_BUTTON_OUT) button->state = LUI_BUTTON_IN;
		else button->state = LUI_BUTTON_OUT;

		/* Dispatch function */
		if (button->func)
		  (*(button->func))(button);
	    }

	    else if (button->group == LUI_BUTTON_NODRAW) {
		/* Dispatch function */
		if (button->func)
		  (*(button->func))(button);
	    }
	    break;
	}
	break;
	
      case ButtonRelease:
	break;
    }
}

/******************************************************************************/
 static
LUI_DrawOptionButton(label, x, y, xs, ys)
    LUI_LABEL *label;
    int x, y, xs, ys;
{    
    LUI_DrawRadioButton(label, x, y, xs, ys);
}

/******************************************************************************/
 static
LUI_DrawRadioButton(label, x, y, xs, ys)
    LUI_LABEL *label;
    int x, y, xs, ys;
{
    LUI_BUTTON *button;

    button = (LUI_BUTTON *)label->owner;

    if (button->state == LUI_BUTTON_IN) {
	if (button->pixmap_in)
	  XCopyArea(LUI_Display, button->pixmap_in, label->window, 
		    LUI_GC_fore, x, y, ys, ys, 0, 0);
	  
    }
    else if (button->state == LUI_BUTTON_OUT) {
	if (button->pixmap_out)
	  XCopyArea(LUI_Display, button->pixmap_out, label->window, 
		    LUI_GC_fore, x, y, ys, ys, 0, 0);
	  
    }
}


/******************************************************************************/
/********** LUI DESTROY LOGIC * BILLH 10-4-91 *********************************/
/******************************************************************************/


/******************************************************************************/
/* added by billh 10-4-91 */
LUI_ButtonPadDestroy(bp)
LUI_BUTTON_PAD *bp;
{
  LUI_BUTTON_PAD *lbp, *nbp;
  int n, m;

  LUI_ButtonFindDestroy(bp);
  lbp = NULL;
  nbp = LUI_ButtonPadHead;
  while (nbp != NULL) {
    if (nbp == bp) {
      /* this is the button pad */
      /* delete from button pad list */
      if (lbp == NULL) LUI_ButtonPadHead = bp->np;
      else lbp->np = bp->np;
      /* delete from event list */
      delete_event(bp->window);
      /* free memory */
      free(bp);
      return;
    }
    lbp = nbp;
    nbp = nbp->np;
  }
  /* not found */
  return;
}


/******************************************************************************/
/* added by billh 10-4-91 */
LUI_ButtonFindDestroy(owner)
LUI_BUTTON_PAD *owner;
{
  LUI_BUTTON *bp, *lbp, *nbp;

GOT_ONE:
  bp = LUI_ButtonHead;
  while (bp != NULL) {
    if (bp->button_pad == owner) {
      LUI_ButtonDestroy(bp);
      goto GOT_ONE;
    }
    bp = bp->np;
  }
  /* not found */
  return;
}


/******************************************************************************/
/* added by billh 10-4-91 */
LUI_ButtonDestroy(bp)
LUI_BUTTON *bp;
{
  LUI_BUTTON *lbp, *nbp;
  int n, m;

  LUI_LabelFindDestroy(bp);
  lbp = NULL;
  nbp = LUI_ButtonHead;
  while (nbp != NULL) {
    if (nbp == bp) {
      /* this is the button */
      /* delete from button list */
      if (lbp == NULL) LUI_ButtonHead = bp->np;
      else lbp->np = bp->np;
      /* delete from event list */
      delete_event(bp->window);
      /* free memory */
      free(bp);
      return;
    }
    lbp = nbp;
    nbp = nbp->np;
  }
  /* not found */
  return;
}

